#!/bin/bash
#
# File(movie)2Genre version 3
# Organize movie files into directories based on the results of the OMDB movie search
#
# Copyright (c) 2009-2018 JF Nutbroek <jfnutbroek@gmail.com>
# Visit http://www.mywebmymail.com for more information
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
# 
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# Usage:
#
# This script should be located in the same directory as the movie files you want to sort
# The script will determine the genre of the file by using the OMDB search engine
# The script will create the genre directory if it does not exists and move the file there
# The file will be tagged with the year of release and the IMDB score, example: 2017 (6) The Movie.mp4
# When the script is finished all files should be placed in the correct directory
# Remaining files need to be moved to the correct directories manually
#
# To run the script in test mode use option -t                      : sh file2genre.sh -t
# To run the script on already processed filenames use option -g    : sh file2genre.sh -g
#

# Get your own API key from http://www.omdbapi.com/apikey.aspx
APIKEY="ab123456"

# Function to clean up old workfiles
clean_up() {
if [ -f ".search.html" ]; then
rm .search.html
fi
if [ -f ".search.txt" ]; then
rm .search.txt
fi
if [ -f ".count" ]; then
rm .count
fi
}

TEST="N"
GENREONLY="N"
if [ "$1" = "-t" ]; then
TEST="Y"
fi
if [ "$2" = "-t" ]; then
TEST="Y"
fi
if [ "$1" = "-g" ]; then
GENREONLY="Y"
fi
if [ "$2" = "-g" ]; then
GENREONLY="Y"
fi

# Get all files in this directory
echo ""
echo " Initializing"
FILENAMES="`find ./ -type f -name '[^.]*' -maxdepth 1 | sed 's/\ /_/g;' | sed 's/\///g;' | sed 's/.\(.*\)/\1/'`"
FILENAMES=`echo $FILENAMES | sed "s/\file2genre.sh//g"`
ELEMENTS=($FILENAMES)
TOTAL=${#ELEMENTS[@]}
COUNTFILE=1
echo " Found ${TOTAL} files to move"

# Determine the genre/category from each filename and move to genre/category directory
for FILENAME in $FILENAMES
do

clean_up
FILE="`echo $FILENAME | sed 's/%/ /g' | sed 's/_/+/g' | sed 's/\.[^\.]*$//'`"
FILE=${FILE// /+}
if [ "$GENREONLY" = "Y" ]; then
FILE=`echo ${FILE:9}`
fi
if [ "$TEST" = "Y" ]; then
echo "Checking: ${FILE}"
echo "http://www.omdbapi.com/?apikey=${APIKEY}&type=movie&t=${FILE}"
fi
curl --user-agent "Opera/9.80 (J2ME/MIDP; Opera Mini/4.2.14912/870; U; id) Presto/2.4.15" --silent -L "http://www.omdbapi.com/?apikey=${APIKEY}&type=movie&t=${FILE}" > .search.html
textutil -convert txt .search.html

RESPONSE=`cat .search.txt | sed 's/{.*Response":"*\([0-9a-zA-Z]*\)"*,*.*}/\1/'`

if [ "$RESPONSE" = "True" ]; then
if [ "$TEST" = "Y" ]; then
        echo "Movie found!"
    fi

    YEAR=`cat .search.txt | sed 's/{.*Year":"*\([0-9a-zA-Z]*\)"*,*.*}/\1/'`
    SIZE=${#YEAR}
    if [ "$SIZE" != "4" ]; then
        YEAR="????"
    fi

    RATING=`cat .search.txt | sed 's/{.*imdbRating":"*\([0-9a-zA-Z]*\)"*.*}/\1/'`
    SIZE=${#RATING}
    if [ "$SIZE" != "1" ]; then
        RATING="?"
    fi
    if [ "$RATING" = "N" ]; then
        RATING="?"
    fi

    GENRE=`cat .search.txt | sed 's/{.*Genre":"*\([0-9a-zA-Z, ]*\)"*,*.*}/\1/' | sed 's/\ //g;'`
    if [ "$TEST" = "Y" ]; then
        echo "All Genres: ${GENRE}"
    fi
    if [[ $GENRE == *","* ]]; then
        IFS=',' read -ra GENRES <<< "$GENRE"
        for i in "${GENRES[@]}"; do
            GENRE=`echo $i`
        done
        if [ "$TEST" = "Y" ]; then
            echo "Genre: ${GENRE}"
        fi
    fi

    if [ "$GENRE" != "" ]; then
        if [ ! -d $GENRE ]; then
            if [ "$TEST" = "N" ]; then
                mkdir $GENRE
            fi
        fi
        FILENAME="`echo $FILENAME | sed 's/%/ /g' | sed 's/_/ /g'`"
        if [ "$TEST" = "N" ]; then
            mv "${FILENAME}" "${GENRE}/${FILENAME}"
            if [ "$GENREONLY" = "N" ]; then
                mv "${GENRE}/${FILENAME}" "${GENRE}/${YEAR} (${RATING}) ${FILENAME}"
            fi
        else
            echo " Testing: Will move ${FILENAME} to ${GENRE}"
            echo " Year & Rating: ${YEAR} (${RATING})"
            echo " Path: ${GENRE}/${YEAR} (${RATING}) ${FILENAME}"
        fi
    fi

else

    if [ "$TEST" = "Y" ]; then
        echo "Unable to find movie"
    fi
fi

# Progress bar
if [ "$1" != "-t" ]; then
PERCENT=$(($COUNTFILE*100/$TOTAL))
BAR=""
DONE=$(($PERCENT/2))
for ((c=0;c<=${DONE};c++))
do
BAR="${BAR}="
done
BAR="${BAR}>"
for ((c=${DONE};c<50;c++))
do
BAR="${BAR}."
done
printf "\r Progress: [ %s%s ] $PERCENT%%" ${BAR}
COUNTFILE=$(($COUNTFILE+1))
fi

done

clean_up
echo " Completed"
echo " Done!"
echo " Move any remaining files manually"
echo ""

exit 0
